# import the necessary packages
import argparse
import cv2
import easygui
import numpy as np
import math
from numpy.linalg import eig, inv
from pylab import *
import matplotlib.pyplot as plt

# initialize the list of reference points
refPt = []
x = []
y = []

def fitEllipse(refPt):
    x = [p[0] for p in refPt]
    y = [p[1] for p in refPt]
    #x = x[:,np.newaxis]
    #y = y[:,np.newaxis]
    x = np.array(x)
    y = np.array(y)
    #x = x[:,np.newaxis]
    #y = y[:,np.newaxis]
    #D =  np.hstack((x*x, x*y, y*y, x, y, np.ones_like(x)))
    D = []
    for i in xrange(x.size):
        D.append([])
        D[i].append(x[i]*x[i])
        D[i].append(x[i]*y[i])
        D[i].append(y[i]*y[i])
        D[i].append(x[i])
        D[i].append(y[i])
        D[i].append(1.0)
    D = np.array(D)
    S = np.dot(D.transpose(),D)
    C = np.zeros([6,6])
    C[0,2] = C[2,0] = 2; C[1,1] = -1
    E, V =  eig(np.dot(inv(S), C))
    n = np.argmax(np.abs(E))
    a = V[:,n]
    return a

def ellipse_center(a):
    b,c,d,f,g,a = a[1]/2, a[2], a[3]/2, a[4]/2, a[5], a[0]
    num = b*b-a*c
    x0=(c*d-b*f)/num
    y0=(a*f-b*d)/num
    return np.array([x0,y0])


def ellipse_angle_of_rotation( a ):
    b,c,d,f,g,a = a[1]/2, a[2], a[3]/2, a[4]/2, a[5], a[0]
    return 0.5*np.arctan(2*b/(a-c))


def ellipse_axis_length( a ):
    b,c,d,f,g,a = a[1]/2, a[2], a[3]/2, a[4]/2, a[5], a[0]
    up = 2*(a*f*f+c*d*d+g*b*b-2*b*d*f-a*c*g)
    down1=(b*b-a*c)*( (c-a)*np.sqrt(1+4*b*b/((a-c)*(a-c)))-(c+a))
    down2=(b*b-a*c)*( (a-c)*np.sqrt(1+4*b*b/((a-c)*(a-c)))-(c+a))
    res1=np.sqrt(up/down1)
    res2=np.sqrt(up/down2)
    return np.array([res1, res2])

def ellipse_angle_of_rotation2( a ):
    b,c,d,f,g,a = a[1]/2, a[2], a[3]/2, a[4]/2, a[5], a[0]
    if b == 0:
        if a < c:
            return 0
        else:
            return np.pi/2
    else: 
        if a < c:
            return np.arctan(2*b/(a-c))/2
        else:
            return np.pi/2 + np.arctan(2*b/(a-c))/2

 
def click_SelectPoint_OR_DrawElipse(event, x, y, flags, param):
    global refPt
    # if the left mouse button was clicked, record the (x, y) coordinates
    if event == cv2.EVENT_LBUTTONDOWN:
        refPt.append((x, y))
        cv2.circle(image,(x,y), 3, (0,0,0), -1)
	# check to see if the left mouse button was released
    elif event == cv2.EVENT_RBUTTONUP:
        xx = [p[0] for p in refPt]
        yy = [p[1] for p in refPt]
        a = fitEllipse(refPt)
        center = ellipse_center(a)
        #phi = ellipse_angle_of_rotation(a)
        phi = ellipse_angle_of_rotation2(a)
        axes = ellipse_axis_length(a)

        print("center = ",  center)
        print("angle of rotation = ",  phi)
        print("axes = ", axes)

        xx = np.array(xx)
        yy = np.array(yy)

        plt.plot(xx, yy, '*')     # given points

        a, b = axes
        arc = 2.0
        R = np.arange(0,arc*np.pi, 0.01)

        xxx = []
        yyy = []

        for i in range(R.size):
            xxx.append(center[0] + a*np.cos(R[i])*np.cos(phi) - b*np.sin(R[i])*np.sin(phi))
            yyy.append(center[1] + a*np.cos(R[i])*np.sin(phi) + b*np.sin(R[i])*np.cos(phi))
        
        px = int (center[0])
        py = int (center[1])
        a = int (axes[0])
        b = int (axes[1])

        cv2.ellipse(image,(px,py),(a,b),phi*180/np.pi,0.0,360.0,(0,0,0),1)


        #plot(xx,yy)
        plt.plot(xxx,yyy, 'r')
        plt.show()

nomFichier = easygui.fileopenbox()

image = cv2.imread(nomFichier,0)

#image = cv2.imread('oeil10degre.pgm',0)
clone = image.copy()
cv2.namedWindow("image")
cv2.setMouseCallback("image", click_SelectPoint_OR_DrawElipse)
 
# keep looping until the 'q' key is pressed
while True:
	# display the image and wait for a keypress
	cv2.imshow("image", image)
	key = cv2.waitKey(1) & 0xFF
 
	# if the 'c' key is pressed, break from the loop
	if key == ord("c"):
		break
 

 
# close all open windows
cv2.destroyAllWindows()